use Test::More tests => 10;

use_ok 'Cog';
use_ok 'Cog::App';
use_ok 'Cog::Base';
use_ok 'Cog::Config';
use_ok 'Cog::FileBrowser';
use_ok 'Cog::Maker';
use_ok 'Cog::Node';
use_ok 'Cog::Plugin';
use_ok 'Cog::Store';
use_ok 'Cog::WebApp';
