# TODO:
# - Review all code
package Cog;
use 5.008003;

our $VERSION = '0.06';

=encoding utf8

=head1 NAME

Cog - The Cog Information Application Framework

=head1 SYNOPSIS

    > cd content/dir
    > cog init
    > cog make
    > cog start

=head1 STATUS

This software is pre-alpha. Don't use it yet.

=head1 DESCRIPTION

Cog lets you turn any directory on your computer into an interactive
application, like a website or a wiki.

This module installs a command line utility called C<cog>. This command
can be used to create and update your application.

=head1 DOCUMENTATION

See L<Cog::Manual> for more information.

=head1 KUDOS

Many thanks to the good people of Strategic Data in Melbourne Victoria
Australia, for supporting me and this project. \o/

=head1 AUTHOR

Ingy döt Net <ingy@cpan.org>

=head1 COPYRIGHT AND LICENSE

Copyright (c) 2010, 2011. Ingy döt Net.

This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

See http://www.perl.com/perl/misc/Artistic.html

=cut
